#define DEBUG false

// RTC Config
#include "RTClib.h"
RTC_DS3231 rtc;

// Strip LED config
#include "Adafruit_NeoPixel.h"
#define LEDPIN 6
#define CNT 24
Adafruit_NeoPixel ring = Adafruit_NeoPixel(CNT, LEDPIN, NEO_GRB + NEO_KHZ800);

// Battery levels
#define C_ALARM (CNT/6)
#define C_WARNING (2*CNT/6)
#define C_FULL (5*CNT/6)
#define BATTERY 0
int pos = 0;
int charge = 0;

// Switch config
#define SWITCHCMD 7
#define ON true
#define OFF false

// Days of week
char daysOfTheWeek[7][12] = {"Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche"};
// Days off
int daysOff[7] = {6, 7};

// Forced start forcée
#define FORCEDSTART 9
bool ForcedStart = OFF;

struct prog {
    int Heure;
    int Minute;
    boolean Etat;
};

struct prog Calendar [] = {
  {  0,  0, OFF },
  {  7, 45, ON },
  {  8,  0, OFF },
  {  9, 45, ON },
  { 10,  0, OFF },
  { 11, 45, ON },
  { 13, 25, OFF },
  { 15, 15, ON },
  { 15, 30, OFF },
  { 17, 20, ON },
  { 17, 45, OFF }
};

boolean _state = false;

const int Day = 1;
const int Month = 2;
const int Year = 3;
const int Hour = 4;
const int Minute = 5;

void setup () {
  Serial.begin(56700);
  while (!Serial);

  ring.begin();
  ring.setBrightness(255);
  
  pinMode(FORCEDSTART, INPUT_PULLUP);
  pinMode(SWITCHCMD, OUTPUT);

  if (! rtc.begin()) {
    Serial.println("!!! STOP !!! Couldn't find RTC");
    while(true){};
  } else {
    Serial.println("- RTC initialized");
  }
  
  if (rtc.lostPower()) {
    Serial.println("RTC lost power, let's set the time!");
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
  Serial.println("- End of Setup");
}

void loop () {
  SOCCheck();
  digitalWrite(SWITCHCMD, checkState());
  Serial.print(".");
  // delay(5000);
}

boolean checkState() {
  int CalendarSize = sizeof(Calendar)/sizeof(*Calendar);
  int daysOffSize = sizeof(daysOff)/sizeof(*daysOff);
  
  DateTime now = rtc.now();
  if(DEBUG) {
    Serial.print("*** Time = ");
    Serial.print(now.hour());
    Serial.print(" H ");
    Serial.print(now.minute());
    Serial.print(" min");
    Serial.println(" ***");
    Serial.print("*** Day = ");
    Serial.print(daysOfTheWeek[now.dayOfTheWeek()-1]);
    Serial.print(" ");
    Serial.print(now.day());
    Serial.print("/");
    Serial.print(now.month());
    Serial.print("/");
    Serial.print(now.year());
    Serial.println(" ***");
  }
  
  // Test from calandar
  for (int i=0; i<CalendarSize; i++) {
    double nowSec = now.hour()*3600+now.minute()*60+now.second();
    double checSec = Calendar[i].Heure*3600+Calendar[i].Minute*60;
    if(DEBUG) {
      Serial.print("Check !!!  Now = "); Serial.print(nowSec); Serial.print("  check = "); Serial.println(checSec);
    }
    if (nowSec > checSec) {
      _state = Calendar[i].Etat;
      if(DEBUG) {
        Serial.print("Bingo !!!"); Serial.print(Calendar[i].Heure); Serial.print(":"); Serial.print(Calendar[i].Minute); Serial.print(" "); Serial.println(Calendar[i].Etat);
      }
    }
  }
  // Test for dayOff
  for (int j=0; j<daysOffSize; j++) {
    if(now.dayOfTheWeek() == daysOff[j] ) {
      _state = false;
      if(DEBUG) {
        Serial.print(now.dayOfTheWeek()); Serial.print("-"); Serial.print(daysOfTheWeek[now.dayOfTheWeek()-1]); Serial.print(": "); Serial.println("  *** This is dayOff !!!");
      }
    }
  }
  // Test for Forced Start
  if(!digitalRead(FORCEDSTART)) {
    _state = true;
    if(DEBUG) {
      Serial.println("  *** FORCED START !!!");
    }
  }
  // Test for SOC minimum
  if(charge<C_WARNING) {
    _state = false;
    if(DEBUG) {
      Serial.println("  *** SOC minimum !!!");
    }
  }
  return _state;
}

void SOCCheck() {
  int value = analogRead(BATTERY);
  //    0 --> 0V
  //  680 --> 10V
  //  970 --> 14,2V
  // 1023 --> 15V
  charge = map(value, 650, 850, 0, CNT);
  if(DEBUG) {
    Serial.print(value);
    Serial.print(" --> ");
    Serial.println(charge);
  }
  for (int i = 0; i < CNT; i++) {
    if (i<charge) {
        if(i<=C_ALARM) {
        ring.setPixelColor((i + pos), 255, 0, 0);
      }
         if(i>C_ALARM & i<=C_WARNING) {
        ring.setPixelColor((i + pos), 255, 128, 0);
      }
         if(i>C_WARNING & i<=C_FULL) {
        ring.setPixelColor((i + pos), 51, 255, 51);
      }
        if(i>C_FULL) {
        ring.setPixelColor((i + pos), 0, 255, 0);
      }
    } else {
      ring.setPixelColor((i + pos), 0, 0, 0);
    }
  }
  ring.show();
}
